#include <QApplication>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QPropertyAnimation>
#include <QLayout>
#include <QPushButton>

class MovableGraphicsPixmapItem : public QObject, public QGraphicsPixmapItem
{
    Q_OBJECT
    Q_PROPERTY( QPointF pos READ pos WRITE setPos )

public:
    MovableGraphicsPixmapItem( const QPixmap& pixmap, QGraphicsItem* parent = 0 )
	: QObject(), QGraphicsPixmapItem( pixmap, parent ) {
    }
};

class HarnessPrivate
{
public:
    HarnessPrivate() 
	: movingAreaSize( QSize( 360, 120 ) ), horizontalOffset( 8 ) {
    }

    QSize movingAreaSize;
    int horizontalOffset;
    QPropertyAnimation* yellowBallAnimation;
};

class Harness : public QWidget
{
    Q_OBJECT
    Q_DECLARE_PRIVATE( Harness )

public:
    Harness();
    ~Harness();

private slots:
    void startAnimation();

private:
    MovableGraphicsPixmapItem* createBallItem( const QString& pixmapFileName );
    HarnessPrivate* d_ptr;
};

Harness::Harness()
    : QWidget( 0 ), d_ptr( new HarnessPrivate )
{
    Q_D( Harness );

    QGraphicsScene* graphicsScene = new QGraphicsScene( 0, 0, d->movingAreaSize.width(), d->movingAreaSize.height() );
    MovableGraphicsPixmapItem* yellowBallItem = createBallItem( ":/images/YellowGlassBall.png" );
    QSize ballSize = yellowBallItem->pixmap().size();
    int ballSpacingY = ( d->movingAreaSize.height() - ballSize.height())  / 2;
    int yellowBallY = ballSpacingY;
    yellowBallItem->setPos( d->horizontalOffset, yellowBallY );
    graphicsScene->addItem( yellowBallItem );

    QGraphicsView* graphicsView = new QGraphicsView();
    graphicsView->setScene( graphicsScene );

    QPushButton* animateButton = new QPushButton( "Animate" );

    QHBoxLayout* buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch();
    buttonLayout->addWidget( animateButton );

    QVBoxLayout* topLayout = new QVBoxLayout;
    topLayout->addWidget( graphicsView );
    topLayout->addLayout( buttonLayout );

    setLayout( topLayout );
    setFixedSize( sizeHint() );

    d->yellowBallAnimation = new QPropertyAnimation( yellowBallItem, "pos", this );
    d->yellowBallAnimation->setDuration( 5 * 1000 );
    int movableBallStartX = d->horizontalOffset;
    int movableBallEndX = d->movingAreaSize.width() - ballSize.width() - d->horizontalOffset;
    d->yellowBallAnimation->setStartValue( QPoint( movableBallStartX, yellowBallY ) );
    d->yellowBallAnimation->setEndValue( QPoint( movableBallEndX, yellowBallY ) );
    d->yellowBallAnimation->setEasingCurve( QEasingCurve::OutBounce );

    connect( animateButton, SIGNAL( clicked() ), SLOT( startAnimation() ) );
}

Harness::~Harness()
{
    delete d_ptr;
}

void Harness::startAnimation()
{
    Q_D( Harness );
    
    d->yellowBallAnimation->start();
}

MovableGraphicsPixmapItem* Harness::createBallItem( const QString& pixmapFileName )
{
    MovableGraphicsPixmapItem* item = new MovableGraphicsPixmapItem( QPixmap( pixmapFileName ) );

    return item;
}

int main( int argc, char** argv )
{
    QApplication app( argc, argv );

    Harness harness;
    harness.show();

    return app.exec();
}

#include "movablegraphicsitem.moc"
